/*
 * File         : DotProduct.C
 * Author       : Han-Ku Lee
 * Created      : Sat Oct 30 18:16:43 1999
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1999/12/20 22:22:28 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1999
 */

#include "hpjava_spmd_adlib_DotProduct.h"
#include "adlib.h"

extern jfieldID hCPPObjID;

/*
 * Class:     hpjava_spmd_adlib_DotProduct
 * Method:    clean
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_hpjava_spmd_adlib_DotProduct_clean
  (JNIEnv *, jclass);

/*
 * Class:     hpjava_spmd_adlib_DotProduct
 * Method:    execute
 * Signature: ([F[F)F
 */
JNIEXPORT jfloat JNICALL Java_hpjava_spmd_adlib_DotProduct_execute
  (JNIEnv* env, jobject jthis, jfloatArray src1, jfloatArray src2)
{
  jfloat result;

  //jint dLength1 = env->GetArrayLength(src1);
  //jint dLength2 = env->GetArrayLength(src2);

  jfloat *jSA1 = env->GetFloatArrayElements(src1, 0);
  jfloat *jSA2 = env->GetFloatArrayElements(src2, 0);

  ((DotProduct<jfloat,jfloat,jfloat> *)env->GetLongField(jthis, hCPPObjID))->execute(&result, jSA1, jSA2);

  env->ReleaseFloatArrayElements(src1, jSA1, JNI_ABORT);
  env->ReleaseFloatArrayElements(src2, jSA2, JNI_ABORT);

  return result;
}

/*
 * Class:     hpjava_spmd_adlib_DotProduct
 * Method:    finalize
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_hpjava_spmd_adlib_DotProduct_finalize
  (JNIEnv* env, jobject jthis)
{
  delete (DotProduct<jfloat,jfloat,jfloat> *)env->GetLongField(jthis, hCPPObjID);
}

/*
 * Class:     hpjava_spmd_adlib_DotProduct
 * Method:    float_constructor
 * Signature: (Lhpjava/lang/DAD;Lhpjava/lang/DAD;I)V
 */
JNIEXPORT void JNICALL Java_hpjava_spmd_adlib_DotProduct_float_1constructor
  (JNIEnv* env, jobject jthis, jobject s1DAD, jobject s2DAD, jint len)
{
  DAD* s1DADPtr = (DAD*)env->GetLongField(s1DAD, hCPPObjID);
  DAD* s2DADPtr = (DAD*)env->GetLongField(s2DAD, hCPPObjID);

  DotProduct<jfloat,jfloat,jfloat> *cppDotProduct 
    = new DotProduct<jfloat,jfloat,jfloat>(s1DADPtr, s2DADPtr);

  if (cppDotProduct == NULL) {
    jclass cError = env->FindClass("java/lang/OutOfMemeoryError");
    env->ThrowNew(cError, "adlib memory allocation failed");
    return;
  }

  env->SetLongField(jthis, hCPPObjID, (long)cppDotProduct);  
}

/*
 * Class:     hpjava_spmd_adlib_DotProduct
 * Method:    init
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_hpjava_spmd_adlib_DotProduct_init
  (JNIEnv* env, jclass jthis)
{
  jclass InterfaceObject_class=env->FindClass("hpjava/lang/InterfaceObject");
  hCPPObjID=env->GetFieldID(InterfaceObject_class, "hCPPObj", "J");    
}
