/*
 * File         : hpjava_spmd_adlib_Shift.C
 * Author       : Guansong Zhang
 * Created      : Tue Nov  4 21:00:00 1997
 * Revision     : $Revision: 1.2 $
 * Updated      : $Date: 1999/12/07 19:03:09 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#include "hpjava_spmd_adlib_Shift.h"
#include "adlib.h"

//DAD * hpjava_lang_DAD_ResolvePtr(JNIEnv *, jobject);
extern jfieldID hCPPObjID;

/*
 * Class:     hpjava_spmd_adlib_Shift
 * Method:    init
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_hpjava_spmd_adlib_Shift_init
  (JNIEnv * env, jclass jthis)
{
  jclass InterfaceObject_class=env->FindClass("hpjava/lang/InterfaceObject");
  hCPPObjID=env->GetFieldID(InterfaceObject_class, "hCPPObj", "J");  
}

/*
 * Class:     hpjava_spmd_adlib_Shift
 * Method:    clean
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_hpjava_spmd_adlib_Shift_clean
  (JNIEnv *, jclass);

Mode int2mode(int i)
{
  switch (i) {
  case 0:
    return CYCL;
    break;

  case 1:
    return EDGE;
    break;
    
  case 2:
    return NONE;
    break;

  default:
    printf("Not defined mode %d\n", i);
    return CYCL;
  }
  return CYCL;
}

/*
 * Class:     hpjava_spmd_adlib_Shift
 * Method:    constructor
 * Signature: (Lhpjava/lang/DAD;Lhpjava/lang/DAD;IIII)V
 */
JNIEXPORT void JNICALL Java_hpjava_spmd_adlib_Shift_constructor
  (JNIEnv *env, jobject jthis, jobject dDAD, jobject sDAD, jint len, jint shift, jint dim, jint mode)
{
  //DAD * dDADPtr = hpjava_lang_DAD_ResolvePtr(env, dDADPtr); 
  DAD * dDADPtr = (DAD*)env->GetLongField(dDAD, hCPPObjID); 
  //DAD * sDADPtr = hpjava_lang_DAD_ResolvePtr(env, sDADPtr); 
  DAD * sDADPtr = (DAD*)env->GetLongField(sDAD, hCPPObjID); 

  Shift *cppShift = new Shift(dDADPtr, sDADPtr, len, shift, dim, int2mode(mode));

  if (cppShift == NULL) {
    jclass cError=env->FindClass("java/lang/OutOfMemeoryError");
    env->ThrowNew(cError,"adlib memory allocation failed");
    return;
  }

  env->SetLongField(jthis, hCPPObjID, (long)cppShift);  

}

/*
 * Class:     hpjava_spmd_adlib_Shift
 * Method:    finalize
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_hpjava_spmd_adlib_Shift_finalize
  (JNIEnv * env, jobject jthis) 
{
  delete (Shift *)env->GetLongField(jthis, hCPPObjID);
}

/*
 * Class:     hpjava_spmd_adlib_Shift
 * Method:    execute
 * Signature: ([IJ[IJ)V
 */
JNIEXPORT void JNICALL Java_hpjava_spmd_adlib_Shift_execute___3IJ_3IJ
  (JNIEnv * env, jobject jthis, jintArray dst, jlong dOffset, jintArray src, jlong sOffset)
{
  jint * jDA=env->GetIntArrayElements(dst, 0);
  jint * jSA=env->GetIntArrayElements(src, 0);

  ((Shift *)env->GetLongField(jthis, hCPPObjID))->execute(jDA+dOffset, jSA+sOffset); 

  env->ReleaseIntArrayElements(dst, jDA, 0);
  env->ReleaseIntArrayElements(src, jSA, JNI_ABORT);
}

/*
 * Class:     hpjava_spmd_adlib_Shift
 * Method:    execute
 * Signature: ([FJ[FJ)V
 */
JNIEXPORT void JNICALL Java_hpjava_spmd_adlib_Shift_execute___3FJ_3FJ
  (JNIEnv * env, jobject jthis, jfloatArray dst, jlong dOffset, jfloatArray src, jlong sOffset)
{
  jfloat * jDA=env->GetFloatArrayElements(dst, 0);
  jfloat * jSA=env->GetFloatArrayElements(src, 0);

  ((Shift *)env->GetLongField(jthis, hCPPObjID))->execute(jDA+dOffset, jSA+sOffset); 

  env->ReleaseFloatArrayElements(dst, jDA, 0);
  env->ReleaseFloatArrayElements(src, jSA, JNI_ABORT);
}

/*
 * Class:     hpjava_spmd_adlib_Shift
 * Method:    execute
 * Signature: ([BJ[BJ)V
 */
JNIEXPORT void JNICALL Java_hpjava_spmd_adlib_Shift_execute___3BJ_3BJ
(JNIEnv * env, jobject jthis, jbyteArray dst, jlong dOffset, jbyteArray src, jlong sOffset)
{
  jbyte * jDA=env->GetByteArrayElements(dst, 0);    
  jbyte * jSA=env->GetByteArrayElements(src, 0);

  ((Shift *)env->GetLongField(jthis, hCPPObjID))->execute(jDA+dOffset, jSA+sOffset); 

  env->ReleaseByteArrayElements(dst, jDA, 0);
  env->ReleaseByteArrayElements(src, jSA, JNI_ABORT);
}

/*
 * Class:     hpjava_spmd_adlib_Shift
 * Method:    execute
 * Signature: ([CJ[CJ)V
 */
JNIEXPORT void JNICALL Java_hpjava_spmd_adlib_Shift_execute___3CJ_3CJ
(JNIEnv * env, jobject jthis, jcharArray dst, jlong dOffset, jcharArray src, jlong sOffset)
{
  jchar * jDA=env->GetCharArrayElements(dst, 0);    
  jchar * jSA=env->GetCharArrayElements(src, 0);

  ((Shift *)env->GetLongField(jthis, hCPPObjID))->execute(jDA+dOffset, jSA+sOffset); 

  env->ReleaseCharArrayElements(dst, jDA, 0);
  env->ReleaseCharArrayElements(src, jSA, JNI_ABORT);
}

/*
 * Class:     hpjava_spmd_adlib_Shift
 * Method:    execute
 * Signature: ([DJ[DJ)V
 */
JNIEXPORT void JNICALL Java_hpjava_spmd_adlib_Shift_execute___3DJ_3DJ
(JNIEnv * env, jobject jthis, jdoubleArray dst, jlong dOffset, jdoubleArray src, jlong sOffset)
{
  jdouble * jDA=env->GetDoubleArrayElements(dst, 0);    
  jdouble * jSA=env->GetDoubleArrayElements(src, 0);

	((Shift *)env->GetLongField(jthis, hCPPObjID))->execute(jDA+dOffset, jSA+sOffset); 

  env->ReleaseDoubleArrayElements(dst, jDA, 0);
  env->ReleaseDoubleArrayElements(src, jSA, JNI_ABORT);
}

/*
 * Class:     hpjava_spmd_adlib_Shift
 * Method:    execute
 * Signature: ([JJ[JJ)V
 */
JNIEXPORT void JNICALL Java_hpjava_spmd_adlib_Shift_execute___3JJ_3JJ
(JNIEnv * env, jobject jthis, jlongArray dst, jlong dOffset, jlongArray src, jlong sOffset)
{
  jlong * jDA=env->GetLongArrayElements(dst, 0);    
  jlong * jSA=env->GetLongArrayElements(src, 0);

  ((Shift *)env->GetLongField(jthis, hCPPObjID))->execute(jDA+dOffset, jSA+sOffset); 

  env->ReleaseLongArrayElements(dst, jDA, 0);
  env->ReleaseLongArrayElements(src, jSA, JNI_ABORT);
}

/*
 * Class:     hpjava_spmd_adlib_Shift
 * Method:    execute
 * Signature: ([SJ[SJ)V
 */
JNIEXPORT void JNICALL Java_hpjava_spmd_adlib_Shift_execute___3SJ_3SJ
(JNIEnv * env, jobject jthis, jshortArray dst, jlong dOffset, jshortArray src, jlong sOffset)
{
  jshort * jDA=env->GetShortArrayElements(dst, 0);    
  jshort * jSA=env->GetShortArrayElements(src, 0);

  ((Shift *)env->GetLongField(jthis, hCPPObjID))->execute(jDA+dOffset, jSA+sOffset); 

  env->ReleaseShortArrayElements(dst, jDA, 0);
  env->ReleaseShortArrayElements(src, jSA, JNI_ABORT);
}

/*
 * Class:     hpjava_spmd_adlib_Shift
 * Method:    execute
 * Signature: ([ZJ[ZJ)V
 */
JNIEXPORT void JNICALL Java_hpjava_spmd_adlib_Shift_execute___3ZJ_3ZJ
(JNIEnv * env, jobject jthis, jbooleanArray dst, jlong dOffset, jbooleanArray src, jlong sOffset)
{
  jboolean * jDA=env->GetBooleanArrayElements(dst, 0);    
  jboolean * jSA=env->GetBooleanArrayElements(src, 0);

  ((Shift *)env->GetLongField(jthis, hCPPObjID))->execute(jDA+dOffset, jSA+sOffset); 

  env->ReleaseBooleanArrayElements(dst, jDA, 0);
  env->ReleaseBooleanArrayElements(src, jSA, JNI_ABORT);
}

